protocol TransVector {
    associatedtype Element        // 付属型の宣言
    var x: Element { get }
    var y: Element { get }
    func transposed() -> Self     // 自分と同じ型を返す関数
    static func +(lhs: Self, rhs: Self) -> Self  // 同じ型同士の演算
}

struct VectorInt: TransVector, CustomStringConvertible {
    typealias Element = Int       // 付属型に Int を設定
    let x, y: Int
    func transposed() -> VectorInt {  // Self の部分は自身の型にする
        return VectorInt(x: self.y, y: self.x)
    }
    static func +(lhs: VectorInt, rhs: VectorInt) -> VectorInt {
        return VectorInt(x: lhs.x + rhs.x, y: lhs.y + rhs.y)
    }
    var description: String { return "[\(x), \(y)]" }
}

let a = VectorInt(x:10, y:3)
print(a.transposed())
let b = VectorInt(x:-1, y:2)
print(a + b)
