protocol SimpleVector {           // ２次元ベクトル型
    associatedtype Element        // 付属型の宣言
    var x : Element { get set }
    var y : Element { get set }
}

struct VectorFloat : SimpleVector {
    typealias Element = Float               // 具体的な型を別名で指定
    var x, y: Float
}
struct VectorDouble : SimpleVector, CustomStringConvertible {
    var x, y: Double                        // 具体的な型は推論される
    var description: String { return "[\(x), \(y)]" }
    init(x: VectorDouble.Element, y: VectorDouble.Element) {
        self.x = x; self.y = y     // 付属型を型として使うことも可能
    }
    init(vectorFloat d: VectorFloat) {
        self.init(x: Double(d.x), y: Double(d.y))
    }
}
struct VectorGrade : SimpleVector, CustomStringConvertible {
    enum Element : String { case A, B, C, D, X }  	// ネスト型を定義
    var x, y: Element
    var description: String { return "[\(x), \(y)]" }
}

var a = VectorFloat(x: 10.0, y: 12.0)
print("x:\(a.x), y:\(a.y)")
let b = VectorDouble(vectorFloat: a)
print(b)
var g = VectorGrade(x:.A, y:.C)
print(g)
