class NamePlate : CustomStringConvertible {
    var name, title: String
    class func makePlates(_ num: Int) -> [NamePlate] {  // 新たに追加
        var bulk = [NamePlate]()
        for _ in 0 ..< num {
            bulk.append( self.init() )  // selfは実行しているクラスを表す
        }
        return bulk
    }
    required init() {                   // required が必要
        name = "出席者"; title = ""
    }
    init?(name:String, title:String) {  // 失敗のあるイニシャライザ
        if name == "" { return nil }    // 名前がないと失敗
        self.name = name
        self.title = title
    }
    var description : String {          // printで表示できる
        if title == "" { return name }
        return "[\(title)] " + name
    }
}

class SpeakerNamePlate : NamePlate {
    required init() {                   // required が必要
        super.init()
        name = ""; title = "講演者"
    }
    override init?(name:String, title:String) {   // 失敗のあるイニシャライザ
        super.init(name:name, title:title)
        // スーパークラスで失敗すると、ここには戻ってこない
        if self.title == "" { self.title = "講演者" }
    }
}

class GuestNamePlate : NamePlate {
    required convenience init() {       // required が必要
        self.init(name:"ゲスト", title:"")
    }
    override init(name:String, title:String) {     // 失敗しない
        if name.isEmpty {                          // name == "" と同じ
            super.init()                           // 失敗のない定義を呼ぶ
            if !title.isEmpty { self.name = title }
        }else {
            super.init(name: name, title: title)!  // 失敗がないことを明示
        }
    }
}

// var plate: NamePlate = SpeakerNamePlate()

print( NamePlate(name:"加藤 恵", title:"主演")! )         // [主演] 加藤 恵
print( SpeakerNamePlate(name:"", title:"企画") ?? "")     // 失敗
print( SpeakerNamePlate(name:"柏木 エリ", title:"")! )    // [講演者] 柏木 エリ
print( GuestNamePlate(name:"", title:"プロデューサー") )
let plates = SpeakerNamePlate.makePlates(5)
print(plates)
