struct Person {
    let name: String    // 変更できない
    var height: Int     // 変更可能
}
struct HighScore {      // ハイスコア
    var player: Person  // 人物
    var score: Int      // 点数
}
class Locker {                          // ロッカー
    var owner: Person                   // 所有者
    var user: Person { return owner }   // 使用者
    init(owner p: Person) { owner = p }
}

var ps = Person(name:"東郷", height:158)
var score = HighScore(player:ps, score:210)
var locker = Locker(owner: ps)

let kp1 = \HighScore.player.height
print(type(of:kp1))                     // WritableKeyPath<HighScore, Int>
let kp2 = \HighScore.player.name
print(type(of:kp2))                     // WritableKeyPath<HighScore, String>
let kp3 = \Locker.owner.height
print(type(of:kp3))                     // ReferenceWritableKeyPath<Locker, Int>
let kp4 = \Locker.user.height
print(type(of:kp4))                     // KeyPath<Locker, Int>

// ただし、nameは変更できないことに注意。
// score[keyPath: kp2] = "鷲尾"

score[keyPath: kp1] = 155               // 代入が可能
print(score[keyPath: kp1])              // 155
locker[keyPath: kp3] += 2               // += 演算子も使える
print(locker[keyPath: kp4])             // 160
