// 全角を半角に変換する
extension String {
    var isHankaku: Bool {
        for ch in self.unicodeScalars {
            if !(0x20 ..< 0x7F ~= ch.value) { return false }
        }
        return true
    }

    static var zenSpaceDouble: Bool = true  // 全角空白を２つの半角空白にする

    func toHankaku() -> String {
        var r = ""
        for ch in self.unicodeScalars {
            var v = ch.value       // 文字コードを得る
            switch v {
            case 0xFF01 ..< 0xFF5F: // 全角文字
                v -= 0xff01 - 0x21  // ASCII文字にする
                r.unicodeScalars.append(Unicode.Scalar(v)!) // ビューを変更
            case 0x3000:            // 全角の空白文字
                r += String.zenSpaceDouble ? "  " : " "
            default:
                r.unicodeScalars.append(ch) // ビューを変更
            }
        }
        return r
    }
}

let orig = "Ａｓｕｋａ\u{3000}2001年１２月０４日"
print(orig)
print(orig.isHankaku)
print("2001-12-04".isHankaku)
String.zenSpaceDouble = false
print(orig.toHankaku())
print(Int("１５４９８".toHankaku())! + 34)
