protocol SimpleVector {           // ２次元ベクトル型
    associatedtype Element        // 付属型の宣言
    var x : Element { get set }
    var y : Element { get set }
}

extension SimpleVector where Element: FloatingPoint { // 要素が実数だったら
    func abs() -> Element {
        return (x * x + y * y).squareRoot()     // ベクトルの絶対値
    }
    func toString() -> String { return "(\(x), \(y))" } // 文字列の表現
}
extension SimpleVector where Element == String { // 要素が文字列だったら
    func toString() -> String { return x + y }   // 文字列の表現
}

struct VectorFloat : SimpleVector {
    var x, y: Float          // 要素として Float を使う
}

let f = VectorFloat(x: 4.0, y: 7.0)
print(f.toString())
print(f.abs())

struct 棋譜 : SimpleVector {
    var x, y: String         // 要素として String を使う
}

let p = 棋譜(x:"４", y:"六")
print(p.toString())
