enum Name {
    case word(String)               // 普通の語
    case withRuby(String, String)   // 振り仮名（ルビ）付き
    static func ==(lhs:Name, rhs:Name) -> Bool {
        switch (lhs, rhs) {
        case let (.word(a), .word(b)),      // ルビがなければ語の比較
            let (.word(a), .withRuby(b,_)),
            let (.withRuby(a,_), .word(b)): return a == b
        case let (.withRuby(a,s), .withRuby(b,t)):
            return a == b && s == t         // 語とルビの比較
        }
    }
}

let a = Name.withRuby("結城", "ゆうき")
let b = Name.withRuby("茅場", "かやば")
print( a == Name.withRuby("勇気", "ゆうき") )
print( b == Name.word("茅場") )

print("------")

// List10-5
extension Name : Equatable { }

let list = [a, b, Name.word("朝田"), Name.word("篠崎")]
if let idx = list.index(of: Name.word("茅場")) {
    print("Index:", idx)
}

// メソッド contains も利用可能
print( list.contains(Name.word("結城")) )
