protocol Dateable {
    var year: Int { get }
    var month: Int { get }
    var day: Int { get }
    var isLeap: Bool { get }    // 追加: うるう年だったら true
}

extension Dateable {
    var isLeap: Bool { // プロトコル本体の宣言に対する既定実装
        return (year % 4 == 0) && (year % 100 != 0 || year % 400 == 0) 
    }
    private static func Zeller(_ y:Int, _ m:Int, _ d:Int) -> Int {
        var y = y, m = m       // 少々トリッキー。変数yは仮引数のyとは違う
        if m < 3 {             // 1,2月だったら
            m += 12; y -= 1    // 前年の13月、14月として計算する
        }
        let leap = y + y / 4 - y / 100 + y / 400
        return (leap + (13 * m + 8) / 5 + d) % 7
    }
    var dayOfWeek: Int {
        return Self.Zeller(year, month, day)
    }
}

struct SimpleDate : Dateable, CustomStringConvertible {
    let year, month, day: Int
    var description: String { return "\(year). \(month). \(day)" }
}

/* 別の実装 */
class Date2 : Dateable, CustomStringConvertible {
    var tm:(Int, Int, Int)
    var year: Int { return tm.0 }
    var month: Int { return tm.1 }
    var day: Int { return tm.2 }
    let isLeap: Bool
    init(_ t:(Int, Int, Int)) {
        let y = t.0
        tm = t
        isLeap = (y % 4 == 0) && (y % 100 != 0 || y % 400 == 0) 
    }
    var description: String { return "\(tm.0). \(tm.1). \(tm.2)" }
}

let d2 = Date2((1994, 2, 1))
print(d2)
print(d2.dayOfWeek)
print(d2.isLeap)

let d3: Dateable = SimpleDate(year:1992, month:7, day:25)
print(d3)
print(d3.dayOfWeek)
print(d3.isLeap)
