// List11-11.swift の一部を変更して
// プロパティリストの読み書きを行うようにしたプログラム例。
// 注意: Linuxでは動作しません。

import Foundation

struct People : Codable, CustomStringConvertible {
    enum CodingKeys : CodingKey { case name0, name1, sex }
    enum Sex : Int { case male, female }    // 性別を表す列挙型
    let name: (String, String)    // ( 姓, 名 )
    let sex: People.Sex?          // 列挙型で性別を表す
    init(name:(String, String), sex: People.Sex? = nil) {
        self.name = name; self.sex = sex
    }
    var description: String {
        var r = "\(name.0) \(name.1)"
        if let s = sex { r += (s == .male) ? "[M]" : "[F]" }
        return r
    }
    func encode(to encoder: Encoder) throws {
        var container = encoder.container(keyedBy: CodingKeys.self)
                // このコンテナがキー付きのエンコードを行う
        try container.encode(name.0, forKey: .name0)
        try container.encode(name.1, forKey: .name1)
                // 値に対応するキーを指定してエンコードを行う
        if let x = sex { // nil の場合はエンコードしない
            try container.encode(x.rawValue, forKey: .sex)
        }
    }
    init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
                // このコンテナがキー付きのデコードを行う
        let n0 = try container.decode(String.self, forKey: .name0)
        let n1 = try container.decode(String.self, forKey: .name1)
                // 型と、対応するキーを指定してデコードを行う
        name = (n0, n1)
        let x = try container.decodeIfPresent(Int.self, forKey: .sex)
        sex = (x == nil) ? nil : People.Sex(rawValue: x!)
                // 整数値から列挙型を得る。デコードできない場合はnil
    }
}
let group = [ People(name:("戸塚", "彩加")),
    People(name:("三浦", "優美子"), sex: .female) ]

let path = URL(fileURLWithPath:"./data_custom.plist")
var encoder = PropertyListEncoder() // エンコーダを得る
encoder.outputFormat = .xml         // XML形式を選択
var decoder = PropertyListDecoder() // デコーダを得る
do {
    let d = try encoder.encode(group)     // groupをエンコード
    try d.write(to:path)                 // データをファイルに保存
    // 上で作ったデータを読み込んで復元する
    let dat = try Data(contentsOf:path)
    let gs = try decoder.decode([People].self, from: dat)
    print(gs)  // 復元したデータを表示
} catch { print(error) }
