enum ConvError : Error {
    case empty
}

func convInt(_ s:String) throws -> Int? {
    if s == "" { throw ConvError.empty }  // 空文字列の時、エラーを発生する
    return Int(s)                         // 整数またはnilを返す
}

for s in [ "41", "Ainz", "" ] {
    // if case let x? = try? convInt(s) も同じ。エラー発生を判断できる。
    if let x = try? convInt(s) {  
        if let n = x { print(n) }    // "41" → 41 を表示
        else { print("nil") }        // "Ainz" → nil を表示
    }else {
        print("Failure")             // "" の時（エラー）はここで処理
    }
}

for s in [ "41", "Ainz", "" ] {
    if case let x?? = try? convInt(s) { // 整数に変換できたかどうかを判断
        print(x)                        // "41" → 41 を表示
    }else {
        print("Failure")                // "Ainz", "" はここで処理
    }
}

for s in [ "41", "Ainz", "" ] {     // ３つの場合が区別できる
    switch try? convInt(s) {
    case let x??: print(x)          // "41" はここで表示
    case nil?: print("nil?")        // "Ainz" はここで表示
    case nil: print("nil")          // "" はここで表示
    }
}
