struct PickForever<T> {
    var list: [T]
    private var index = 0
    init<S>(_ a: S) where S: Sequence, T == S.Element {
        list = [T](a)                        // 初期値はシークエンスで指定
    }
    mutating func pick() -> T {              // 要素を順番に取り出す
        if index >= list.count { index = 0 } // 末尾を過ぎたら先頭に戻る
        let rtn = list[index]
        index += 1
        return rtn
    }
}

extension PickForever where T == String {
    mutating func concatenate(_ n: Int) -> String {
        guard list.count > 0, n > 0 else { return "" }
        var buf = self.pick()
        for _ in 1 ..< n { buf += ":" + self.pick() }
        return buf
    }
}

var c = PickForever(["初雪", "吹雪", "白雪", "深雪"])
print(c.concatenate(2))
print(c.concatenate(7))
