protocol SimpleVector {               // ２次元ベクトル型
    associatedtype Element            // 付属型の宣言
    var x : Element { get set }
    var y : Element { get set }
}

struct Vector<T> : SimpleVector {
    typealias Element = T   // 付属型をパラメータで指定された型で定義
    var x, y: Element       // 格納型プロパティ
}

// プロトコルSimpleVectorの拡張定義。Elementが Equatableに適合している場合のみ有効
extension SimpleVector where Self: Equatable, Element: Equatable { }

// パラメータがEquatableに適合している場合について、== 演算子を定義
func ==<T>(lhs:T, rhs:T) -> Bool
        where T: SimpleVector, T.Element: Equatable {
    return lhs.x == rhs.x && lhs.y == rhs.y
}

let a = Vector<Double>(x:16.0, y:5.0)
var b = Vector<Double>(x:10.0, y:5.0)
print( a == b ) // false: 比較できている
b.x += 6.0
print( a == b ) // true

typealias Item = (String, Int)
let t1 = Vector<Item>(x:("年齢", 18), y:("身長", 165))
let t2 = Vector<Item>(x:("年齢", 18), y:("身長", 165))
// print( t1 == t2 )  // Itemが比較できないので、t1, t2 も比較できない
