class GeoPoint<Element> {
    var latitude, longitude: Element          // 緯度, 経度
    required init(lat:Element, lon:Element) { // 必須イニシャライザ
        self.latitude = lat;  self.longitude = lon
    }
    func toString() -> String { return "\(latitude), \(longitude)" }
}

class GPSPoint<T:FloatingPoint> : GeoPoint<T>, CustomStringConvertible {
    var description: String { return "GPSPoint: " + toString() }
    override func toString() -> String { // 実数に適用可能な定義
        let ns = latitude.sign == .plus
                ? "N\(latitude)" : "S\(-latitude)"   // 北緯と南緯
        let ew = longitude.sign == .plus
                ? "E\(longitude)" : "W\(-longitude)" // 東経と西経
        return ns + ", " + ew   // 例. N36.04844, E138.11203
    }
    func copy() -> Self { // 動的にクラスを調べる
        return type(of:self).init(lat:latitude, lon:longitude)
    }
}

class ViewPoint : GPSPoint<Double> {
    let name: String  // プロパティを追加
    required convenience init(lat:Double, lon:Double) {  // 必須
        self.init(name:"(none)", lat:lat, lon:lon)
    }
    required init(name:String, lat:Double, lon:Double) {
        self.name = name
        super.init(lat:lat, lon:lon)
    }
    override func copy() -> Self { // 動的にクラスを調べる
        let stype = type(of:self)
        return stype.init(name:name, lat:latitude, lon:longitude)
    }
    override var description: String {
        return "ViewPoint: \(name), " + toString()
    }
}

var g0 = GPSPoint<Float>(lat:35.70040, lon:139.77217)
var g1 = g0.copy()
print(g1)
print(g0 === g1)
var g2 = ViewPoint(name:"松本城", lat:36.238064, lon:137.968457)
var g3 = g2.copy()
print(g3)
print(g2 === g3)
