extension WordReader: Sequence, IteratorProtocol {
    typealias Element = String
    typealias Iterator = WordReader
    func next() -> Element? { return self.getWord() } // イテレータの機能
    func makeIterator() -> Iterator { return self }   // シークエンスに必須
}

main : do {
    guard let words = WordReader(open:"text.txt") else {
        print("ファイルがオープンできません")
        break main
    }
    let dic = words.lazy                    // lazyが必要
                   .map{ $0.lowercased() }  // 文字列を英小文字に
                   .reduce(into: [:]) {
        (d: inout [String:Int], key: String) in
        d[key, default:0] += 1                   // 文字列の出現頻度を調べる
    }
    for elm in dic.sorted(by:{ $0.value > $1.value }).prefix(10) {
        print("\"\(elm.0)\"", elm.1)
    } // 出現頻度の多いものから10個を表示
}
