import Cocoa

@objc class Flipper : NSObject {
    var bitmap: NSBitmapImageRep
    @objc init?(data: Data) { // 画像として認識できなければ失敗
        guard let imgrep = NSBitmapImageRep(data: data) else { return nil }
        bitmap = imgrep
        super.init()
    }
    @objc func flip(options opt: FlipOpt) {
        if opt == [] { return } // 変換の必要なし
        var buf = [UnsafeMutablePointer<UInt8>?](repeating:nil, count:5)
        bitmap.getBitmapDataPlanes(&buf)
        let pw = bitmap.pixelsWide               // 水平方向のピクセル数
        let ss = (bitmap.bitsPerPixel + 7) >> 3  // pixelごとのバイト数
        let rawlen = bitmap.bytesPerRow          // 実際の１行のバイト数
        let high = bitmap.pixelsHigh             // 垂直方向のピクセル数
        if opt.contains(.Vertical) {
            let copr = BlockReverser(width:high, bytes:rawlen)
            for plane in buf {
                guard let pl = plane else { break }
                copr.reverse(pl)
            } // 画像全体を、行単位で逆順にする
        }
        if opt.contains(.Horizontal) {
            let copr = BlockReverser(width:pw, bytes:ss)
            for plane in buf {
                guard let pl = plane else { break }
                for h in 0 ..< high {
                    copr.reverse(pl + h * rawlen)
                } // 最初の行から順番に反転させてゆくだけ
            }
        }
    }
    @objc func dataForJPEG() -> Data? {
        // JPEGのオプション。圧縮の程度と、透明色を置き換える色の指定
        typealias PropKey = NSBitmapImageRep.PropertyKey
        let prop: [PropKey:Any] = [ PropKey.compressionFactor: 0.5,
            PropKey.fallbackBackgroundColor: NSColor.white ]
        guard let dat = bitmap.representation(
            using:.jpeg, properties: prop) else { return nil }
        return dat
    }
}
